# -*- coding: utf-8 -*-

import time
from threading import Thread
import logging

from mamonsu.lib.const import Template


class Plugin(object):

    # plugin interval run
    Interval = 60

    _thread = None
    _sender = False
    _enabled = True
    # for all childs
    _child = True

    # const
    DELTA = Template.DELTA
    GRAPH_TYPE = Template.GRAPH_TYPE
    VALUE_TYPE = Template.VALUE_TYPE
    UNITS = Template.UNITS

    DELTA_SPEED = Template.DELTA.speed_per_second
    DELTA_CHANGE = Template.DELTA.simple_change

    def __init__(self, config):
        self.config = config
        self.log = logging.getLogger(
            self.__class__.__name__.upper())
        self.sender = None
        self.last_error_text = ''

    @classmethod
    def childs(self):
        childs = []
        for klass in self.__subclasses__():
            if klass._child:
                childs.append(klass)
            childs.extend(klass.childs())
        return childs

    def start(self):
        self._thread = Thread(target=self._loop)
        self._thread.daemon = True
        self._thread.start()

    def is_alive(self):
        if self._thread is not None:
            return self._thread.is_alive()
        return False

    def run(self, sender):
        return None

    def is_sender(self):
        return self._sender

    def is_enabled(self):
        return self._enabled

    def update_sender(self, sender):
        self.sender = sender

    def items(self, template):
        return None

    def graphs(self, template):
        return None

    def triggers(self, template):
        return None

    def discovery_rules(self, template):
        return None

    def _log_exception(self, e):
        name = e.__class__.__name__
        self.last_error_text = 'Plugin exception [{0}]: {1}'.format(name, e)
        self.log.error(self.last_error_text)

    def _loop(self):
        while(True):
            last_start = time.time()
            try:
                self.run(self.sender)
            except Exception as e:
                self._log_exception(e)
                return
            sleep_time = self.Interval - int(time.time() - last_start)
            if sleep_time > 0:
                time.sleep(sleep_time)
            else:
                self.log.error(
                    'Timeout: {0}s'.format(int(time.time() - last_start)))
                return



import os
from ._connection import Connection


class Pool(object):

    ExcludeDBs = ['template0', 'template1', 'postgres']

    def __init__(self):
        self.connections = {}

    def query(self, query, database=None):
        if database is None:
            database = os.environ.get('PGDATABASE')
        self.__install_connection(database)
        return self.connections[database].query(query)

    def is_extension_installed(self, ext, database):
        result = self.query('select count(*) from pg_catalog.pg_extension\
            where extname = "{0}"'.format(ext), database)
        return (int(result[0][0])) == 1

    def databases(self):
        result, databases = self.query('select datname from \
            pg_catalog.pg_database'), []
        for row in result:
            if row[0] not in self.ExcludeDBs:
                databases.append(row[0])
        return databases

    def __install_connection(self, database):
        conn = self.connections.get(database)
        if conn is None:
            self.connections[database] = Connection(database)

Pooler = Pool()



# -*- coding: utf-8 -*-

from mamonsu.plugins.pgsql.plugin import PgsqlPlugin as Plugin
from .pool import Pooler


class BgWriter(Plugin):

    def __init__(self, config):
        super(BgWriter, self).__init__(config)
        self.CheckpointsReq = self.config.fetch(
            'postgres', 'max_checkpoints_req', int)

    Items = [
        # key, zbx_key, description,
        #    ('graph name', color, side), units, delta

        ('checkpoints_timed', 'checkpoints[checkpoints_timed]',
            'checkpoints: by timeout',
            ('PostgreSQL checkpoints', '00CC00', 0),
            Plugin.UNITS.none, Plugin.DELTA.simple_change),

        ('checkpoints_req', 'checkpoints[checkpoints_req]',
            'checkpoints: required',
            ('PostgreSQL checkpoints', 'CC0000', 0),
            Plugin.UNITS.none, Plugin.DELTA.simple_change),

        ('checkpoint_write_time', 'checkpoint[write_time]',
            'checkpoint: write time',
            ('PostgreSQL checkpoints', '0000CC', 1),
            Plugin.UNITS.ms, Plugin.DELTA.simple_change),

        ('checkpoint_sync_time', 'checkpoint[checkpoint_sync_time]',
            'checkpoint: sync time',
            ('PostgreSQL checkpoints', '000000', 1),
            Plugin.UNITS.ms, Plugin.DELTA.simple_change),

        ('buffers_checkpoint', 'bgwriter[buffers_checkpoint]',
            'bgwriter: buffers written during checkpoints',
            ('PostgreSQL bgwriter', 'CCCC00', 1),
            Plugin.UNITS.bytes, Plugin.DELTA.simple_change),

        ('buffers_clean', 'bgwriter[buffers_clean]',
            'bgwriter: buffers written',
            ('PostgreSQL bgwriter', '0000CC', 1),
            Plugin.UNITS.bytes, Plugin.DELTA.simple_change),

        ('maxwritten_clean', 'bgwriter[maxwritten_clean]',
            'bgwriter: number of bgwriter stopped by max write count',
            ('PostgreSQL bgwriter', '777777', 0),
            Plugin.UNITS.ms, Plugin.DELTA.simple_change),

        ('buffers_backend', 'bgwriter[buffers_backend]',
            'bgwriter: buffers written directly by a backend',
            ('PostgreSQL bgwriter', 'CC0000', 1),
            Plugin.UNITS.bytes, Plugin.DELTA.simple_change),

        ('buffers_backend_fsync', 'bgwriter[buffers_backend_fsync]',
            'bgwriter: times a backend execute its own fsync',
            ('PostgreSQL bgwriter', 'CC00CC', 0),
            Plugin.UNITS.none, Plugin.DELTA.simple_change),

        ('buffers_alloc', 'bgwriter[buffers_alloc]',
            'bgwriter: buffers allocated',
            ('PostgreSQL bgwriter', '00CC00', 1),
            Plugin.UNITS.bytes, Plugin.DELTA.simple_change)
    ]

    def run(self, zbx):
        params = [x[0] for x in self.Items]
        result = Pooler.query('select {0} from pg_stat_bgwriter'.format(
            ', '.join(params)))
        for idx, val in enumerate(result[0]):
            key, val = 'pgsql.{0}'.format(
                self.Items[idx][1]), int(val)
            zbx.send(key, val, self.Items[idx][5])
        del params, result

    def items(self, template):
        result = ''
        for item in self.Items:
            result += template.item({
                'key': 'pgsql.{0}'.format(item[1]),
                'name': 'PostgreSQL {0}'.format(item[2]),
                'value_type': self.VALUE_TYPE.numeric_unsigned,
                'units': item[4]
            })
        return result

    def graphs(self, template):
        name = 'PostgreSQL bgwriter'
        items = []
        for item in self.Items:
            if not item[3] is None:
                if item[3][0] == name:
                    items.append({
                        'key': 'pgsql.{0}'.format(item[1]),
                        'color': item[3][1],
                        'yaxisside': item[3][2]
                    })
        graph = {'name': name, 'items': items}
        result = template.graph(graph)

        name = 'PostgreSQL checkpoints'
        items = []
        for item in self.Items:
            if not item[3] is None:
                if item[3][0] == name:
                    items.append({
                        'key': 'pgsql.{0}'.format(item[1]),
                        'color': item[3][1],
                        'yaxisside': item[3][2]
                    })
        graph = {'name': name, 'items': items}
        result += template.graph(graph)

        return result

    def triggers(self, template):
        return template.trigger({
            'name': 'PostgreSQL required checkpoints occurs to '
            'frequently on {HOSTNAME}',
            'expression': '{#TEMPLATE:pgsql.checkpoints[checkpoints_req]'
            '.last()}&gt;' + str(self.CheckpointsReq)
        })



# -*- coding: utf-8 -*-

import optparse
import socket
import os
import logging
import sys
import glob
import codecs

import mamonsu.lib.platform as platform
from mamonsu.plugins.pgsql.checks import is_conn_to_db
from mamonsu.lib.default_config import DefaultConfig

from mamonsu import __version__
from mamonsu.lib.plugin import Plugin
from mamonsu.lib.zbx_template import *

if platform.PY2:
    import ConfigParser as configparser
else:
    import configparser


class Config(DefaultConfig):

    def __init__(self, override_config_filename=None):

        parser = optparse.OptionParser(
            usage='%prog [-c] [-p]',
            version='%prog {0}'.format(__version__))

        group = optparse.OptionGroup(
            parser,
            'Start options')
        group.add_option(
            '-c',
            dest='config_filename',
            default=None,
            help='Path to config file')
        group.add_option(
            '-p',
            dest='pid',
            default=None,
            help='Path to pid file')
        parser.add_option_group(group)

        group = optparse.OptionGroup(
            parser,
            'Example options',
            'Export default options')
        group.add_option(
            '-w',
            dest='write_config_file',
            default=None,
            help='Write default config to file and exit')
        parser.add_option_group(group)

        group = optparse.OptionGroup(
            parser,
            'Export to zabbix',
            'Export zabbix template options')
        group.add_option(
            '-e',
            dest='template_file',
            default=None,
            help='Write template to file and exit')
        group.add_option(
            '-t',
            dest='template',
            default='PostgresPro-{0}'.format(sys.platform.title()),
            help='Generated template name')
        group.add_option(
            '-a',
            dest='application',
            default='App-PostgresPro-{0}'.format(sys.platform.title()),
            help='Application for generated template')
        parser.add_option_group(group)

        args, _ = parser.parse_args()

        config = configparser.ConfigParser()

        config.add_section('plugins')
        config.set('plugins', 'directory', str(None))

        config.add_section('template')
        config.set('template', 'name', args.template)
        config.set('template', 'application', args.application)

        config.add_section('postgres')
        config.set('postgres', 'enabled', str(True))
        config.set('postgres', 'user', Config.default_user())
        config.set('postgres', 'password', str(Config.default_pgpassword()))
        config.set('postgres', 'database', Config.default_db())
        config.set('postgres', 'host', Config.default_host())
        config.set('postgres', 'port', str(Config.default_port()))
        config.set('postgres', 'application_name', str(Config.default_app()))
        config.set('postgres', 'max_checkpoints_req', '5')
        config.set('postgres', 'query_timeout', '10')

        config.add_section('log')
        config.set('log', 'file', str(None))
        config.set('log', 'level', 'INFO')
        config.set(
            'log',
            'format',
            '[%(levelname)s] %(asctime)s - %(name)s\t-\t%(message)s')

        config.add_section('zabbix')
        config.set('zabbix', 'enabled', str(True))
        config.set('zabbix', 'client', socket.gethostname())
        config.set('zabbix', 'address', '127.0.0.1')
        config.set('zabbix', 'port', str(10051))

        config.add_section('metric_log')
        config.set('metric_log', 'enabled', str(False))
        config.set('metric_log', 'directory', str(None))
        config.set('metric_log', 'max_size_mb', '1024')

        config.add_section('sender')
        config.set('sender', 'queue', str(300))

        config.add_section('health')
        config.set('health', 'uptime', str(60 * 10))
        config.set('health', 'cache', str(80))

        config.add_section('disk_sizes')
        config.set('disk_sizes', 'percent_free', str(10))
        config.set('disk_sizes', 'inode_percent_free', str(10))

        config.add_section('system')
        config.set('system', 'enabled', str(True))

        # override config file name
        if override_config_filename is not None:
            args.config_filename = override_config_filename

        self.config = config
        if args.config_filename and not os.path.isfile(args.config_filename):
            sys.exit(1)
        else:
            if args.config_filename is not None:
                self.config.read(args.config_filename)
        self._apply_log_setting()
        self._apply_environ()
        self._load_additional_plugins()

        if args.write_config_file is not None:
            with open(args.write_config_file, 'w') as fd:
                config.write(fd)
                sys.exit(0)

        if args.template_file is not None:
            plugins = []
            for klass in Plugin.childs():
                plugins.append(klass(self))
            template = ZbxTemplate(args.template, args.application)
            with codecs.open(args.template_file, 'w', 'utf-8') as templatefile:
                templatefile.write(template.xml(plugins))
                sys.exit(0)

        if args.pid is not None:
            try:
                with open(args.pid, 'w') as pidfile:
                    pidfile.write(str(os.getpid()))
            except Exception as e:
                logging.error('Can\'t write pid file, error: %s'.format(e))
                sys.exit(2)

        self._override_auto_variables()

    def fetch(self, sec, key, klass=None, raw=False):
        try:
            if klass == float:
                return self.config.getfloat(sec, key)
            if klass == int:
                return self.config.getint(sec, key)
            if klass == bool:
                return self.config.getboolean(sec, key)
            if self.config.get(sec, key, raw=raw) == 'None':
                return None
            return self.config.get(sec, key, raw=raw)
        except KeyError:
            return None

    def _apply_environ(self):
        os.environ['PGUSER'] = self.fetch('postgres', 'user')
        if self.fetch('postgres', 'password'):
            os.environ['PGPASSWORD'] = self.fetch('postgres', 'password')
        os.environ['PGHOST'] = self.fetch('postgres', 'host')
        os.environ['PGPORT'] = str(self.fetch('postgres', 'port'))
        os.environ['PGDATABASE'] = self.fetch('postgres', 'database')
        os.environ['PGTIMEOUT'] = self.fetch('postgres', 'query_timeout')
        os.environ['PGAPPNAME'] = self.fetch('postgres', 'application_name')

    def _apply_log_setting(self):
        logging.basicConfig(
            format=self.fetch('log', 'format', raw=True),
            filename=self.fetch('log', 'file'),
            level=self.get_logger_level(self.fetch('log', 'level')))

    def _load_additional_plugins(self):

        directory = self.fetch('plugins', 'directory')
        if directory is None:
            return

        if not os.path.isdir(directory):
            logging.error("Can't find directory: %s", directory)
            sys.exit(3)

        sys.path.append(directory)
        logging.info(
            'Import module \'%s\' from directory %s',
            os.path.basename(directory),
            directory)

        try:
            for filename in glob.glob(os.path.join(directory, '*.py')):
                if not os.path.isfile(filename):
                    continue
                # /dir/filename.py => filename.py
                filename = os.path.basename(filename)
                if filename.startswith('_'):
                    continue
                # filename.py => filename
                filename, _ = os.path.splitext(filename)
                logging.info(
                    'Import plugin \'%s\' from module \'%s\'',
                    filename,
                    os.path.basename(directory))
                __import__(filename)
        except Exception as e:
            logging.error(
                'Can\'t load module: %s', e)
            sys.exit(3)

    def _override_auto_variables(self):
        self._override_auto_host()

    def _override_auto_host(self):

        def test_db(self, host_pre):
            if is_conn_to_db(
                host=host_pre,
                db=self.fetch('postgres', 'database'),
                port=str(self.fetch('postgres', 'port')),
                user=self.fetch('postgres', 'user'),
                    paswd=self.fetch('postgres', 'password')):
                self.config.set('postgres', 'host', host_pre)
                self._apply_environ()
                return True
            return False

        host = self.fetch('postgres', 'host')
        port = str(self.fetch('postgres', 'port'))
        if host == 'auto' and platform.LINUX:
            logging.debug('Host set to auto, test variables')
            if test_db(self, '/tmp/.s.PGSQL.{0}'.format(port)):
                return
            if test_db(self, '/var/run/postgresql/.s.PGSQL.{0}'.format(port)):
                return
            if test_db(self, '127.0.0.1'):
                return
            #  не выходим, так как ожидаем коннекта до localhost
            self.config.set('postgres', 'host', 'localhost')
            self._apply_environ()



# -*- coding: utf-8 -*-

import sys
import logging

from mamonsu import __version__ as mamonsu_version
from mamonsu.sysinfo.linux import SysInfoLinux


class SystemInfo(SysInfoLinux):

    def __init__(self, args):
        sudo = False if (args.disable_sudo is False) else True
        super(SystemInfo, self).__init__(use_sudo=sudo)
        self.args = args

    def printable_info(self):

        def format_header(info):
            return "\n###### {0:20} ###########################\n".format(info)

        def format_out(key, val):
            return "{0:40}|    {1}\n".format(key, val)

        out = ''
        out += format_header('Report')
        out += format_out('Version', mamonsu_version)
        out += format_out('Platform', sys.platform)
        out += format_out('Python', sys.version_info)
        out += format_header('System')
        out += format_out('Date', self.date)
        out += format_out('Host', self.hostname)
        out += format_out('Uptime', self.uptime_raw)
        out += format_out('System', self.dmi_info['TOTAL'])
        out += format_out('Serial', self.dmi_info['SERIAL'])
        out += format_out('Release', self.release)
        out += format_out('Kernel', self.kernel)
        out += format_out('Arch', 'CPU = {0}, OS = {1}'.format(
            self.cpu_arch, self.os_arch))
        out += format_out('Virt', self.virtualization)
        out += format_header('Processors')
        out += format_out('Total', self.cpu_model['_TOTAL'])
        out += format_out('Speed', self.cpu_model['speed'])
        out += format_out('Model', self.cpu_model['model'])
        out += format_out('Cache', self.cpu_model['cache'])
        out += format_header('Memory')
        out += format_out('Total', self._humansize(self.meminfo['_TOTAL']))
        out += format_out('Cached', self._humansize(self.meminfo['_CACHED']))
        out += format_out('Dirty', self._humansize(self.meminfo['_DIRTY']))
        if 'vm.dirty_ratio' in self.sysctl:
            if 'vm.dirty_background_ratio' in self.sysctl:
                out += format_out('Dirty ratio', '{0} {1}'.format(
                     self.sysctl['vm.dirty_ratio'],
                     self.sysctl['vm.dirty_background_ratio']))
        if 'vm.dirty_bytes' in self.sysctl:
            if 'vm.dirty_background_bytes' in self.sysctl:
                out += format_out('Dirty bytes', '{0} {1}'.format(
                     self.sysctl['vm.dirty_bytes'],
                     self.sysctl['vm.dirty_background_bytes']))
        out += format_out('Swap', self._humansize(self.meminfo['_SWAP']))
        if 'vm.swappiness' in self.sysctl:
            out += format_out('Swappines', self.sysctl['vm.swappiness'])
        out += format_header('System settings')
        for k in self.systemd['_main']:
            out += format_out(k, self.systemd['_main'][k])
        out += format_header('Mount')
        out += self.df_raw + "\n"
        out += format_header('Disks')
        for disk in self.block_info:
            out += format_out(disk, 'Scheduler: {0} Queue: {1}'.format(
                self.block_info[disk]['scheduler'],
                self.block_info[disk]['nr_requests']))
        out += format_header('IOstat')
        out += self.iostat_raw + "\n"
        out += format_header('LVM')
        out += self.vgs_raw + "\n"
        out += self.lvs_raw + "\n"
        out += format_header('Raid')
        if not self.is_empty(self.raid):
            for raid in self.raid:
                out += format_out('Controller', raid)
        return out

    def store_raw(self):
        def format_out(info, val):
            return "# {0} ##################################\n{1}\n".format(
                info, val)
        out = format_out('SYSCTL', self.sysctl['_RAW'])
        out += format_out('DMESG', self.dmesg_raw)
        out += format_out('LSPCI', self.lspci_raw)
        out += format_out('CPUINFO', self.cpu_model['_RAW'])
        out += format_out('MEMINFO', self.meminfo['_RAW'])
        out += format_out('DMIDECODE', self.dmi_raw)
        out += format_out('DF', self.df_raw)
        out += format_out('MOUNT', self.mount_raw)
        out += format_out('MDSTAT', self.mdstat_raw)
        out += format_out('IOSTAT', self.iostat_raw)
        out += format_out('LVS', self.lvs_raw)
        out += format_out('VGS', self.vgs_raw)
        return out

    def collect(self):
        info = self.printable_info()
        logging.error("\n{0}\n".format(self.store_raw()))
        return info

    _suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']

    def _humansize(self, nbytes):
        if nbytes == 0:
            return '0 B'
        i = 0
        while nbytes >= 1024 and i < len(self._suffixes)-1:
            nbytes /= 1024.
            i += 1
        f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
        return '%s %s' % (f, self._suffixes[i])



# -*- coding: utf-8 -*-

import os
import re
import glob
import logging

from mamonsu.sysinfo.linux_shell import Shell

NA = 'N/A'
UNKNOWN = 'Unknown'

VIRT_VMWARE = 'VMWare'
VIRT_QEMU = 'QEmu'
VIRT_KVM = 'KVM'
VIRT_MICROSOFT_VIRTUALPC = 'MS-VirtualPC'
VIRT_MICROSOFT_HYPERV = 'Hyper-V'
VIRT_VIRTUALBOX = 'VirtualBox'
VIRT_XEN = 'Xen'
VIRT_PARALLELS = 'Parallels'

VIRT_VZ = 'OpenVZ/Virtuozzo'
VIRT_LXC = 'LXC'
VIRT_DOCKER = 'Docker'

RAID_LSI = 'LSI Logic MegaRAID SAS'
RAID_HP_SMART = 'HP Smart Array'
RAID_ADAPTEC = 'AACRAID'
RAID_3WARE = '3Ware'
RAID_FUSION_SAS = 'Fusion-MPT SAS'


class SysInfoLinux(object):

    @classmethod
    def is_empty(self, val):
        if val is None or val == NA or val == '':
            return True
        return False

    def __init__(self, use_sudo=True):
        self._cache = {}
        logging.debug('Use sudo: {0}'.format(use_sudo))
        self.use_sudo = use_sudo

    def __getattr__(self, name):

        def remember(self, name, value):
            self._cache[name] = value
            return self._cache[name]

        if name in self._cache:
            return self._cache[name]
        try:
            if name == 'hostname':
                return remember(self, name, self._shell_out('uname -n'))
            if name == 'date':
                return remember(self, name, self._date())
            if name == 'sysctl':
                return remember(self, name, self._sysctl())
            if name == 'dmesg_raw':
                return remember(self, name, self._dmesg_raw())
            if name == 'kernel':
                return remember(self, name, self._shell_out('uname -r'))
            if name == 'uptime_raw':
                return remember(self, name, self._shell_out('uptime'))
            if name == 'mount_raw':
                return remember(self, name, self._shell_out('mount'))
            if name == 'iostat_raw':
                return remember(self, name, self._shell_out(
                        'iostat -x -N -m 1 2', timeout=3))
            if name == 'df_raw':
                return remember(self, name, self._shell_out('df -h -P'))
            if name == 'mdstat_raw':
                return remember(self, name, self._read_file('/proc/mdstat'))
            if name == 'lvs_raw':
                return remember(self, name, self._shell_out('lvs', sudo=True))
            if name == 'vgs_raw':
                return remember(self, name, self._shell_out(
                    'vgs -o vg_name,vg_size,vg_free', sudo=True))
            if name == 'cpu_arch':
                return remember(self, name, self._os_arch())
            if name == 'cpu_arch':
                return remember(self, name, self._cpu_arch())
            if name == 'dmi_raw':
                return remember(self, name, self._shell_out(
                    'dmidecode', sudo=True))
            if name == 'dmi_info':
                return remember(self, name, self._dmi_info())
            if name == 'cpu_model':
                return remember(self, name, self._cpu_model())
            if name == 'meminfo':
                return remember(self, name, self._meminfo())
            if name == 'virtualization':
                return remember(self, name, self._virtualization())
            if name == 'release':
                return remember(self, name, self._release())
            if name == 'raid':
                return remember(self, name, self._raid())
            if name == 'block_info':
                return remember(self, name, self._block_info())
            if name == 'systemd':
                return remember(self, name, self._systemd())
        except KeyError:
            return UNKNOWN

    def _shell_out(self, cmd, timeout=1, sudo=False):
        sudo = sudo and self.use_sudo
        shell = Shell(cmd, timeout=timeout, sudo=sudo)
        sudo_cmd = 'sudo {0}'.format(cmd) if sudo else cmd
        logging.debug('Shell "{0}" status: {1}'.format(sudo_cmd, shell.status))
        if shell.status == 0:
            return shell.stdout
        return NA

    def is_sudo_working(self):
        return not self.is_empty(
            self._shell_out('echo sudo is worked', sudo=True))

    def _uniq(self, values):
        output = []
        seen = set()
        for value in values:
            if value not in seen:
                output.append(value)
                seen.add(value)
        return output

    def _read_file(self, file):
        result = NA
        if os.path.isfile(file):
            try:
                result = open(file, 'r').read()
            except Exception as e:
                logging.debug('File "{0}" read error: {1}'.format(file, e))
                pass
        return result

    def _cpu_arch(self):
        if os.path.isfile('/proc/cpuinfo'):
            try:
                with open('/proc/cpuinfo', 'r') as content_file:
                    content = content_file.read()
                    if re.search(r' lm ', content):
                        return '64-bit'
                    else:
                        return '32-bit'
            except:
                pass
        return NA

    def _os_arch(self):
        shell = Shell('getconf LONG_BIT')
        if shell.status == 0:
            if re.search('64', shell.stdout):
                return '64-bit'
            if re.search('32', shell.stdout):
                return '32-bit'
        return NA

    def _date(self):
        result = ''
        shell = Shell("date -u +'%F %T UTC'")
        if shell.status == 0:
            result += shell.stdout
        shell = Shell("date +'%Z %z'")
        if shell.status == 0:
            result = "{0} (local TZ:{1})".format(result, shell.stdout)
        return result

    def _sysctl(self):
        result = {'_RAW': NA}
        shell = Shell('sysctl -a')
        if not shell.status == 0:
            return result
        result['_RAW'] = shell.stdout
        for out in shell.stdout.split("\n"):
            try:
                k, v = out.split(" = ")
                result[k] = v
            except:
                continue
        return result

    def _dmesg_raw(self):
        shell = Shell('journalctl -k -n 1000')
        if shell.status == 0:
            return shell.stdout
        shell = Shell('dmesg')
        if shell.status == 0:
            return shell.stdout
        result = self._read_file('var/log/dmesg')
        return result

    def _dmidecode(self, key):
        return self._shell_out('dmidecode -s \'{0}\''.format(key), sudo=True)

    def _dmi_info(self):
        result = {}
        result['vendor'] = self._dmidecode('system-manufacturer')
        result['product'] = self._dmidecode('system-product-name')
        result['version'] = self._dmidecode('system-version')
        result['chassis'] = self._dmidecode('chassis-type')
        result['SERIAL'] = self._dmidecode('system-serial-number')
        result['TOTAL'] = '{0}; {1}; {2} ({3})'.format(
            result['vendor'], result['product'],
            result['chassis'], result['version'])
        return result

    def _cpu_model(self):

        def fetch_first(reg, info):
            val = re.search(reg, info, re.M)
            if val is not None:
                return val.group(1)
            else:
                return NA

        info, result = self._read_file('/proc/cpuinfo'), {'_RAW': NA}
        if self.is_empty(info):
            return result
        result['_RAW'] = info

        result['virtual'] = len(
            re.findall(r'(^|\n)processor', info))
        result['physical'] = len(self._uniq(
            re.findall(
                r'^physical id\s+\:\s+(\d+)', info, re.M)))
        cores = re.search(
            r'^cpu cores\s+\:\s+(\d+)', info, re.M)
        if cores is not None:
            result['cores'] = int(cores.group(1))
        else:
            result['cores'] = 0
        if result['physical'] == 0:
            result['physical'] = result['virtual']
        result['hyperthreading'] = False
        if result['cores'] > 0:
            if result['cores'] < result['virtual']:
                result['hyperthreading'] = True
        result['model'] = fetch_first(r'model name\s+\:\s+(.*)$', info)
        result['cache'] = fetch_first(r'cache size\s+\:\s+(.*)$', info)
        result['speed'] = fetch_first(
            r'^cpu MHz\s+\:\s+(\d+\.\d+)$', info) + ' MHz'
        result['_TOTAL'] = 'physical = {0}, cores = {1}, '\
            'virtual = {2}, hyperthreading = {3}'.format(
                result['physical'], result['cores'],
                result['virtual'], result['hyperthreading']
        )
        return result

    def _meminfo(self):

        data, result = self._read_file('/proc/meminfo'), {}
        result['_RAW'] = NA
        result['_TOTAL'] = NA
        result['_SWAP'] = NA
        result['_CACHED'] = NA
        result['_DIRTY'] = NA
        if self.is_empty(data):
            return result

        result['_RAW'] = data
        for info in re.findall(r'^(\S+)\:\s+(\d+)\s+kB$', data, re.M):
            result[info[0]] = int(info[1])*1024
        if 'MemTotal' in result:
            result['_TOTAL'] = result['MemTotal']
        if 'SwapTotal' in result:
            result['_SWAP'] = result['SwapTotal']
        if 'Cached' in result:
            result['_CACHED'] = result['Cached']
        if 'Dirty' in result:
            result['_DIRTY'] = result['Dirty']

        return result

    def _virtualization(self):

        result = self._shell_out('systemd-detect-virt')
        if result == 'qemu':
            return VIRT_QEMU
        if result == 'kvm':
            return VIRT_KVM
        if result == 'vmware':
            return VIRT_VMWARE
        if result == 'microsoft':
            return VIRT_MICROSOFT_HYPERV
        if result == 'oracle':
            return VIRT_VIRTUALBOX
        if result == 'xen':
            return VIRT_XEN
        if result == 'parallels':
            return VIRT_PARALLELS
        if result == 'openvz':
            return VIRT_VZ
        if result == 'lxc':
            return VIRT_LXC
        if result == 'docker':
            return VIRT_DOCKER

        if os.path.isfile('/proc/user_beancounters'):
            return VIRT_VZ

        dmit_product = self.dmi_info['product']
        if re.search(r'VMware', dmit_product, re.I):
            return VIRT_VMWARE
        if re.search(r'VirtualBox', dmit_product, re.I):
            return 'VirtualBox'
        if re.search(r'Bochs', dmit_product, re.I):
            return VIRT_QEMU

        if re.search(
                r'Manufacturer\: Microsoft Corporation', self.dmi_raw, re.I):
            return VIRT_MICROSOFT_VIRTUALPC
        if re.search(r'domU', self.dmi_raw, re.I):
            return VIRT_XEN

        if re.search(r'vmware', self.dmesg_raw, re.I):
            return VIRT_VMWARE
        if re.search(r'vmxnet', self.dmesg_raw, re.I):
            return VIRT_VMWARE
        if re.search(r'paravirtualized kernel on vmi', self.dmesg_raw, re.I):
            return VIRT_VMWARE
        if re.search(r'Xen virtual console', self.dmesg_raw, re.I):
            return VIRT_XEN
        if re.search(r'paravirtualized kernel on xen', self.dmesg_raw, re.I):
            return VIRT_XEN
        if re.search(r'qemu', self.dmesg_raw, re.I):
            return VIRT_QEMU
        if re.search(r'paravirtualized kernel on KVM', self.dmesg_raw, re.I):
            return VIRT_KVM
        if re.search(r'VBOX', self.dmesg_raw, re.I):
            return VIRT_VIRTUALBOX
        if re.search(r'hd.: Virtual .., ATA.*drive', self.dmesg_raw, re.I):
            return VIRT_MICROSOFT_VIRTUALPC

        return NA

    def _release(self):
        result = self._shell_out('lsb_release -ds')
        if not self.is_empty(result):
            return result

        for file in ['/etc/fedora-release', '/etc/redhat-release',
                     '/etc/system-release']:
            if os.path.isfile(file):
                try:
                    with open(file, 'r') as f:
                        result = f.read()
                    return result
                except:
                    pass
        if os.path.isfile('/etc/debian_version'):
            try:
                with open('/etc/debian_version', 'r') as f:
                    result = f.read()
                    return 'Debian-based version {0}'.format(result)
            except:
                pass
        return NA

    def _block_info(self):
        result = {}
        for block in glob.glob('/sys/block/*'):
            data, disk = {}, block.replace('/sys/block/', '')
            if re.search('(ram|loop)', disk):
                continue
            scheduler = '{0}/queue/scheduler'.format(block)
            nr_requests = '{0}/queue/nr_requests'.format(block)
            try:
                if os.path.isfile(scheduler):
                    data['scheduler'] = open(scheduler, 'r').read().strip()
                if os.path.isfile(nr_requests):
                    data['nr_requests'] = open(nr_requests, 'r').read().strip()
                data['fdisk'] = self._shell_out(
                    'fdisk -l /dev/{0}'.format(disk), sudo=True)
            except:
                continue
            result[disk] = data
        return result

    def _raid(self):
        controllers = []
        if not self.is_empty(self.lspci_raw):
            if re.search(
                r'RAID bus controller: LSI Logic / Symbios Logic MegaRAID SAS',
                    self.lspci_raw, re.I):
                controllers.append(RAID_LSI)
            if re.search(
                r'RAID bus controller: LSI Logic / Symbios Logic LSI MegaSAS',
                    self.lspci_raw, re.I):
                controllers.append(RAID_LSI)
            if re.search(
                r'Fusion-MPT SAS',
                    self.lspci_raw, re.I):
                controllers.append(RAID_FUSION_SAS)
            if re.search(
                r'RAID bus controller: LSI Logic / Symbios Logic Unknown',
                    self.lspci_raw, re.I):
                controllers.append(RAID_LSI)
            if re.search(
                r'RAID bus controller: Adaptec AAC-RAID',
                    self.lspci_raw, re.I):
                controllers.append(RAID_ADAPTEC)
            if re.search(
                r'3ware [0-9]* Storage Controller',
                    self.lspci_raw, re.I):
                controllers.append(RAID_3WARE)
            if re.search(
                r'Hewlett-Packard Company Smart Array',
                    self.lspci_raw, re.I):
                controllers.append(RAID_HP_SMART)
            if re.search(
                r'Hewlett-Packard Company Smart Array',
                    self.lspci_raw, re.I):
                controllers.append(RAID_HP_SMART)
        if not self.is_empty(self.dmesg_raw):
            if re.search(r'scsi[0-9].*: .*megaraid', self.dmesg_raw, re.I):
                controllers.append(RAID_LSI)
            if re.search(r'Fusion MPT SAS', self.dmesg_raw):
                controllers.append(RAID_FUSION_SAS)
            if re.search(r'scsi[0-9].*: .*aacraid', self.dmesg_raw, re.I):
                controllers.append(RAID_ADAPTEC)
            if re.search(
                r'scsi[0-9].*: .*3ware [0-9]* Storage Controller',
                    self.dmesg_raw, re.I):
                controllers.append(RAID_3WARE)
        if len(controllers) == 0:
            return NA
        return controllers

    def _systemd(self):
        result = {'_main': {}, 'logind': {}}
        data = self._read_file('/etc/systemd/logind.conf')
        if os.path.isfile('/etc/systemd/logind.conf'):
            try:
                with open('/etc/systemd/logind.conf', 'r') as f:
                    for line in f:
                        data = line.split('=')
                        if not len(data) == 2:
                            continue
                        k, v = data[0].replace('#', ''), data[1]
                        result['logind'][k] = v
                if 'RemoveIPC' in result['logind']:
                    result['_main']['RemoveIPC'] = \
                        result['logind']['RemoveIPC']
            except:
                pass
        return result



# -*- coding: utf-8 -*-
import os
import threading
import logging
from .pg8000 import connect


class Connection(object):

    def __init__(self, database):
        self.database = database
        self.lock = threading.Lock()
        self.log = logging.getLogger('PGSQL')
        self.conn = None
        self.query_completed_succ = False

    def _conn_string(self):
        db = self.database
        host = os.environ.get('PGHOST')
        return 'host={0} db={1}'.format(host, db)

    def query(self, query):
        self.lock.acquire()
        try:
            self.log.debug(
                '[{0}] Run: "{1}"'.format(
                    self._conn_string(), query))
            self._check_connect()
            self.query_completed_succ = False
            cursor = self.conn.cursor()
            cursor.execute(query)
            result = cursor.fetchall()
            cursor.close()
        finally:
            self.lock.release()

        self.query_completed_succ = True
        return result

    def _close(self):
        if self.conn is not None:
            self.log.debug(
                '[{0}] Closing...'.format(
                    self._conn_string()))
            try:
                self.conn.close()
            except Exception as e:
                self.log.error(
                    '[{0}] Closing error: {1}'.format(
                        self._conn_string(), e))

    def _check_connect(self):
        if not self.query_completed_succ:
            self.log.debug(
                '[{0}] Connecting...'.format(
                    self._conn_string()))
            self._close()
            host, unix_sock = os.environ.get('PGHOST'), None
            if host.startswith('/'):
                unix_sock, host = host, None
            self.conn = connect(
                user=os.environ.get('PGUSER'),
                password=os.environ.get('PGPASSWORD'),
                unix_sock=unix_sock,
                host=host,
                port=int(os.environ.get('PGPORT') or 5432),
                database=self.database,
                application_name=os.environ.get('PGAPPNAME')
            )
            self.log.debug(
                '[{0}] Connected!'.format(
                    self._conn_string()))
            self.conn.autocommit = True

            self.log.debug(
                '[{0}] Set statement timeout...'.format(
                    self._conn_string()))
            query_timeout = int(os.environ.get('PGTIMEOUT') or 1)
            cur = self.conn.cursor()
            cur.execute('set statement_timeout to {0}'.format(
                query_timeout * 1000))
            cur.close()



# -*- coding: utf-8 -*-

from mamonsu.plugins.pgsql.plugin import PgsqlPlugin as Plugin
from .pool import Pooler


class Xlog(Plugin):

    TriggerLagMoreThen = 360

    def run(self, zbx):
        # recovery
        result = Pooler.query('select pg_is_in_recovery()')
        if str(result[0][0]) == 't' or str(result[0][0]) == 'True':
            lag = Pooler.query('select extract(epoch from now() \
                - pg_last_xact_replay_timestamp())')
            zbx.send('pgsql.replication_lag[sec]', float(lag[0][0]))
        else:
            # xlog location
            result = Pooler.query("select pg_xlog_location_diff\
                (pg_current_xlog_location(),'0/00000000')")
            zbx.send('pgsql.wal.write[]', float(result[0][0]), self.DELTA_SPEED)

    def items(self, template):
        return template.item({
            'name': 'PostgreSQL: streaming replication lag in seconds',
            'key': 'pgsql.replication_lag[sec]'
        }) + template.item({
            'name': 'PostgreSQL: wal write speed',
            'key': 'pgsql.wal.write[]',
            'units': Plugin.UNITS.bytes
        })

    def graphs(self, template):
        result = template.graph({
            'name': 'PostgreSQL write-ahead log generation speed',
            'items': [
                {'color': 'CC0000',
                    'key': 'pgsql.wal.write[]'}]})
        result += template.graph({
            'name': 'PostgreSQL replication lag in second',
            'items': [
                {'color': 'CC0000',
                    'key': 'pgsql.replication_lag[sec]'}]})
        return result

    def triggers(self, template):
        return template.trigger({
            'name': 'PostgreSQL streaming lag to high '
            'on {HOSTNAME} (value={ITEM.LASTVALUE})',
            'expression': '{#TEMPLATE:pgsql.replication_lag[sec].last'
            '()}&gt;' + str(self.TriggerLagMoreThen)
        })



# -*- coding: utf-8 -*-
import sys
import logging

import mamonsu.lib.platform as platform
from mamonsu.plugins.pgsql.pg8000.core import ProgrammingError
from mamonsu.plugins.pgsql.pool import Pooler
from mamonsu.sysinfo.linux import SysInfoLinux as SysInfo


class AutoTunePgsl(object):

    def __init__(self, args):

        if not self._is_connection_work():
            logging.error('Can\'t connect to PostgreSQL')
            sys.exit(5)

        self.args = args
        self.sys_info = SysInfo()

        self._memory()
        self._auto_vacuum()
        self._bgwriter()
        self._configure_pgbadger()

        self._reload_config()

    def _memory(self):
        if platform.WINDOWS:
            logging.info('No memory tune for windows')
            return

        sysmemory = self.sys_info.meminfo['_TOTAL']
        if sysmemory == 0:
            return

        self._run_query(
            "alter system set shared_buffers to '{0}';".format(
                self._humansize_and_round_bytes(sysmemory/4)))
        self._run_query(
            "alter system set effective_cache_size to '{0}';".format(
                self._humansize_and_round_bytes(3*sysmemory/4)))
        self._run_query(
            "alter system set work_mem to '{0}';".format(
                self._humansize_and_round_bytes(sysmemory/100)))
        self._run_query(
            "alter system set maintenance_work_mem to '{0}';".format(
                self._humansize_and_round_bytes(sysmemory/10)))

    def _auto_vacuum(self):
        self._run_query(
            "alter system set autovacuum_max_workers to 20;")
        self._run_query(
            "alter system set autovacuum_analyze_scale_factor to 0.01;")
        self._run_query(
            "alter system set autovacuum_vacuum_scale_factor to 0.02;")
        self._run_query(
            "alter system set vacuum_cost_delay to 1;")

    def _bgwriter(self):
        self._run_query(
            "alter system set bgwriter_delay to 10;")
        self._run_query(
            "alter system set bgwriter_lru_maxpages to 800;")

    def _configure_pgbadger(self):
        if self.args.pgbadger is not None:
            return
        self._run_query(
            "alter system set logging_collector to on;")
        self._run_query(
            "alter system set log_filename to 'postgresql-%%a.log';")
        self._run_query(
            "alter system set log_checkpoints to on;")
        self._run_query(
            "alter system set log_connections to on;")
        self._run_query(
            "alter system set log_disconnections to on;")
        self._run_query(
            "alter system set log_lock_waits to on;")
        self._run_query(
            "alter system set log_temp_files to 0;")
        self._run_query(
            "alter system set log_autovacuum_min_duration to 0;")
        self._run_query(
            "alter system set log_line_prefix to "
            "'%%t [%%p]: [%%l-1] db=%%d,user=%%u,app=%%a,client=%%h ';")

    def _reload_config(self):
        if self.args.reload_config is not None:
            return
        self._run_query('select pg_catalog.pg_reload_conf();')

    def _is_connection_work(self):
        try:
            Pooler.query('select 1')
            return True
        except Exception as e:
            logging.error('Test query error: {0}'.format(e))
            return False

    def _run_query(self, query='', exit_on_fail=True):
        if self.args.dry_run:
            logging.info('dry run (query):\t{0}'.format(
                query.replace('%%', '%')))
            return
        try:
            Pooler.query(query)
        except ProgrammingError as e:
            if '{0}'.format(e) == 'no result set':
                return
        except Exception as e:
            logging.error('Query {0} error: {1}'.format(query, e))
            if exit_on_fail:
                sys.exit(6)

    def _humansize_and_round_bytes(self, nbytes):
        suffixes = ['', 'kB', 'MB', 'GB', 'TB']
        if nbytes < 1024:
            return str(nbytes)
        i = 0
        while nbytes >= 1024 and i < len(suffixes)-1:
            nbytes /= 1024.
            i += 1
        f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
        return '%s%s' % (int(round(float(f))), suffixes[i])



# -*- coding: utf-8 -*-

import os
import subprocess
import time

# cache value
sudoWorking = None


def is_sudo_working():
    global sudoWorking
    if os.getuid() == 0:
        sudoWorking = True
        return
    if sudoWorking is None:
        sudoWorking = False
        p = subprocess.Popen(
            'sudo true',
            shell=True,
            stdin=None,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            close_fds=True)
        exec_time = 0
        while p.poll() is None:
            time.sleep(0.1)
            exec_time += 0.1
            if exec_time >= 1:
                return
        if p.returncode == 0:
            sudoWorking = True
    return sudoWorking


class Shell(object):

    # exit status of timeout code
    TimeoutCode = -1
    # local var
    _sudo_result = None

    def __init__(self, cmd, timeout=10, sudo=False):
        self.status = self.TimeoutCode
        self.cmd, self._real_command, self.sudo = cmd, cmd, sudo
        if sudo and is_sudo_working():
            self._real_command = 'sudo {0}'.format(cmd)
        self.stdout, self.stderr = '', ''
        self.wait_time, self.exec_time = timeout, 0
        p = subprocess.Popen(
            self._real_command,
            shell=True,
            stdin=None,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            close_fds=True)
        while p.poll() is None:
            time.sleep(0.1)
            self.exec_time += 0.1
            line = p.stdout.read()
            if not line == '':
                self.stdout += line.decode('utf-8')
            line = p.stderr.read()
            if not line == '':
                self.stderr += line.decode('utf-8')
            if self.wait_time > 0 and self.exec_time >= self.wait_time:
                return
        self.status = p.returncode
        for line in p.stdout.readlines():
            self.stdout += line.decode('utf-8')
        for line in p.stderr.readlines():
            self.stderr += line.decode('utf-8')
        self.stdout = self.stdout.strip()
        self.stderr = self.stderr.strip()



# -*- coding: utf-8 -*-

from .pg8000 import connect


def is_conn_to_db(host, db, port, user, paswd):
    host, unix_sock = host, None
    if host.startswith('/'):
        unix_sock, host = host, None
    try:
        conn = connect(
            user=user,
            password=paswd,
            unix_sock=unix_sock,
            host=host,
            port=port,
            database=db)
        cur = conn.cursor()
        cur.execute('select 1')
        cur.close()
    except:
        return False
    else:
        return True



# -*- coding: utf-8 -*-

import os
import logging
import mamonsu.lib.platform as platform


class PgsqlConfig(object):

    @staticmethod
    def default_user():
        user = os.environ.get('PGUSER') or 'postgres'
        return user

    @staticmethod
    def default_pgpassword():
        password = os.environ.get('PGPASSWORD')
        return password

    @staticmethod
    def default_host():
        if platform.WINDOWS:
            host = os.environ.get('PGHOST') or 'localhost'
        if platform.LINUX:
            host = os.environ.get('PGHOST') or 'auto'
        return host

    @staticmethod
    def default_port():
        port = int(os.environ.get('PGPORT') or 5432)
        return port

    @staticmethod
    def default_app():
        app = os.environ.get('PGAPPNAME') or 'mamonsu'
        return app

    @staticmethod
    def default_db():
        database = os.environ.get('PGDATABASE') or os.environ.get('PGUSER')
        database = database or 'postgres'
        return database


class DefaultConfig(PgsqlConfig):

    @staticmethod
    def get_logger_level(level):
        result = logging.INFO
        level = level.upper()
        if level == 'DEBUG':
            return logging.DEBUG
        if level == 'WARNING' or level == 'WARN':
            return logging.WARN
        return result

    @staticmethod
    def default_report_path():
        if platform.WINDOWS:
            return 'c:\\report.txt'
        if platform.LINUX:
            return '/tmp/report.txt'



import re

from mamonsu.plugins.system.plugin import SystemPlugin as Plugin


class ProcStat(Plugin):

    # alert fork-rate
    ForkRate = 500
    # /proc/stat all cpu line
    re_stat = re.compile(
        'cpu\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)')

    ProcessItems = [
        # key, zbx_key, name, delta, color, side
        ('procs_running', 'processes[running]',
            'in state running', Plugin.DELTA.as_is, 'CC0000', 0),
        ('procs_blocked', 'processes[blocked]',
            'in state blocked', Plugin.DELTA.as_is, '00CC00', 0),
        ('processes', 'processes[forkrate]',
            'forkrate', Plugin.DELTA.speed_per_second, '0000CC', 1),
    ]

    CpuItems = [
        # key, zbx_key, name, delta, color, side
        (1, 'cpu[user]',
            'by normal programs and daemons',
            Plugin.DELTA.speed_per_second, '0000CC', 0),
        (2, 'cpu[nice]',
            'by nice(1)d programs',
            Plugin.DELTA.speed_per_second, 'CC00CC', 0),
        (3, 'cpu[system]',
            'by the kernel in system activities',
            Plugin.DELTA.speed_per_second, 'CC0000', 0),
        (4, 'cpu[idle]',
            'Idle CPU time',
            Plugin.DELTA.speed_per_second, '00CC00', 0),
        (5, 'cpu[iowait]',
            'waiting for I/O operations',
            Plugin.DELTA.speed_per_second, 'CCCC00', 0),
        (6, 'cpu[irq]',
            'handling interrupts',
            Plugin.DELTA.speed_per_second, '777777', 0),
        (7, 'cpu[softirq]',
            'handling batched interrupts',
            Plugin.DELTA.speed_per_second, '000077', 0),
    ]

    def run(self, zbx):
        with open('/proc/stat', 'r') as f:
            for line in f:
                data = line.split()
                # parse processes
                for item in self.ProcessItems:
                    if data[0] == item[0]:
                        value = int(data[1])
                        zbx.send(
                            'system.{0}'.format(item[1]), value, item[3])
                        break
                # parse cpu
                if data[0] == 'cpu':
                    m = self.re_stat.match(line)
                    if m is None:
                        continue
                    for item in self.CpuItems:
                        value = m.group(item[0])
                        zbx.send(
                            'system.{0}'.format(item[1]), int(value), item[3])

    def items(self, template):
        result = ''
        for item in self.ProcessItems:
            result += template.item({
                'name': 'Processes: {0}'.format(item[2]),
                'key': 'system.{0}'.format(item[1])
            })
        for item in self.CpuItems:
            result += template.item({
                'name': 'CPU time spent {0}'.format(item[2]),
                'key': 'system.{0}'.format(item[1])
            })
        return result

    def graphs(self, template):
        items = []
        for item in self.ProcessItems:
            items.append({
                'key': 'system.{0}'.format(item[1]),
                'color': item[4],
                'yaxisside': item[5]
            })
        graphs = template.graph({'name': 'Processes overview', 'items': items})
        items = []
        for item in self.CpuItems:
            items.append({
                'key': 'system.{0}'.format(item[1]),
                'color': item[4],
                'yaxisside': item[5]
            })
        graphs += template.graph({
            'name': 'CPU time spent',
            'items': items, 'type': self.GRAPH_TYPE.stacked})
        return graphs

    def triggers(self, template):
        return template.trigger({
            'name': 'Process fork-rate to '
            'frequently on {HOSTNAME}',
            'expression': '{#TEMPLATE:system.processes[forkrate]'
            '.last()}&gt;' + str(self.ForkRate)
        })



import codecs
import mamonsu
from os import path
from setuptools import setup, find_packages


def long_description():
    filespath = path.dirname(path.realpath(__file__))
    with codecs.open(path.join(filespath, 'README.rst'), encoding='utf8') as f:
        return f.read()

setup(
    name='mamonsu',
    version=mamonsu.__version__,
    packages=find_packages(),
    description=mamonsu.__description__,
    long_description=long_description(),
    classifiers=mamonsu.__classifiers__,
    keywords=mamonsu.__keywords__,
    author=mamonsu.__author__,
    author_email=mamonsu.__author_email__,
    url=mamonsu.__url__,
    license=mamonsu.__licence__,
    entry_points={
        'console_scripts': [
            'mamonsu=mamonsu.lib.supervisor:start'
        ],
    },
    zip_safe=True,
)



# -*- coding: utf-8 -*-
import json
import time
from mamonsu.lib.plugin import Plugin


class Sender():

    def __init__(self):
        self._senders = []
        self._last_values = {}

    def update_senders(self, senders=[]):
        self._senders = senders

    def send(self, key, value, delta=None, host=None, clock=None):

        if clock is None:
            clock = int(time.time())

        if delta is not None:
            is_float = isinstance(value, float)
            is_int = isinstance(value, int)
            if is_float or is_int:
                hash_key = '{0}.{1}'.format(host, key)
                if hash_key not in self._last_values:
                    self._last_values[hash_key] = (value, clock)
                    return
                else:
                    last_value = self._last_values[hash_key][0]
                    last_time = self._last_values[hash_key][1]
                    self._last_values[hash_key] = (value, clock)
                    if delta == Plugin.DELTA.speed_per_second:
                        value = float(value - last_value)/(clock - last_time)
                    if delta == Plugin.DELTA.simple_change:
                        value = float(value - last_value)
                    if is_int:
                        value = int(value)

        for sender in self._senders:
            if sender.is_enabled():
                sender.send(key, value, host, clock)

    # helpers
    def json(self, val):
        return json.dumps(val)



from collections import namedtuple


class _template(object):
    # item delta
    _delta = namedtuple(
        'DELTA', ['as_is', 'speed_per_second', 'simple_change'])
    DELTA = _delta(0, 1, 2)
    # grap type
    _graph_type = namedtuple(
        'graph_type', ['normal', 'stacked', 'pie', 'exploded'])
    GRAPH_TYPE = _graph_type(0, 1, 2, 3)
    # value_type
    _value_type = namedtuple(
        'value_type',
        ['numeric_float', 'character', 'log',
            'numeric_unsigned', 'text'])
    VALUE_TYPE = _value_type(0, 1, 2, 3, 4)
    # item unit
    _item_type = namedtuple(
        'item_type', ['none', 'bytes', 'ms', 'uptime', 'percent'])
    UNITS = _item_type(None, 'b', 'ms', 'uptime', '%')


Template = _template()
